package #
Locale::Codes::Language_Retired;

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'deprecate_codes' is run.
#    Generated on: Tue Dec  1 14:45:28 EST 2015

use strict;
require 5.006;
use warnings;
use utf8;

our($VERSION);
$VERSION='3.37';

$Locale::Codes::Retired{'language'}{'alpha-2'}{'code'} = {
   q(in) => q(Indonesian),
   q(iw) => q(Hebrew),
   q(ji) => q(Yiddish),
   q(jw) => q(Javanese),
   q(mo) => q(Moldavian),
};

$Locale::Codes::Retired{'language'}{'alpha-3'}{'code'} = {
   q(aam) => q(Aramanik),
   q(adp) => q(Adap),
   q(aue) => q(=/Kx'au//'ein),
   q(ayx) => q(Ayi (China)),
   q(baz) => q(Tunen),
   q(bjd) => q(Bandjigali),
   q(bjq) => q(Southern Betsimisaraka Malagasy),
   q(bmy) => q(Bemba (Democratic Republic of Congo)),
   q(bxx) => q(Borna (Democratic Republic of Congo)),
   q(byy) => q(Buya),
   q(ccq) => q(Chaungtha),
   q(cka) => q(Khumi Awa Chin),
   q(daf) => q(Dan),
   q(dap) => q(Nisi (India)),
   q(dha) => q(Dhanwar (India)),
   q(djl) => q(Djiwarli),
   q(dkl) => q(Kolum So Dogon),
   q(dwl) => q(Walo Kumbe Dogon),
   q(dzd) => q(Daza),
   q(elp) => q(Elpaputih),
   q(emo) => q(Emok),
   q(gbc) => q(Garawa),
   q(gfx) => q(Mangetti Dune !Xung),
   q(ggr) => q(Aghu Tharnggalu),
   q(gio) => q(Gelao),
   q(gti) => q(Gbati-ri),
   q(hrr) => q(Horuru),
   q(ibi) => q(Ibilo),
   q(ilw) => q(Talur),
   q(ime) => q(Imeraguen),
   q(izi) => q(Izi-Ezaa-Ikwo-Mgbo),
   q(jar) => q(Jarawa (Nigeria)),
   q(kbf) => q(Kakauhua),
   q(kdv) => q(Kado),
   q(kgh) => q(Upper Tanudan Kalinga),
   q(koj) => q(Sara Dunjo),
   q(kpp) => q(Paku Karen),
   q(kwq) => q(Kwak),
   q(kxe) => q(Kakihum),
   q(kzh) => q(Kenuzi-Dongola),
   q(lcq) => q(Luhu),
   q(leg) => q(Lengua),
   q(lii) => q(Lingkhim),
   q(lmm) => q(Lamam),
   q(meg) => q(Mea),
   q(mgx) => q(Omati),
   q(mhh) => q(Maskoy Pidgin),
   q(mja) => q(Mahei),
   q(mld) => q(Malakhel),
   q(mnt) => q(Maykulan),
   q(mwd) => q(Mudbura),
   q(mwj) => q(Maligo),
   q(myq) => q(Forest Maninka),
   q(nbf) => q(Naxi),
   q(nbx) => q(Ngura),
   q(nln) => q(Durango Nahuatl),
   q(nlr) => q(Ngarla),
   q(nnx) => q(Ngong),
   q(noo) => q(Nootka),
   q(oun) => q(!O!ung),
   q(pbz) => q(Palu),
   q(pcr) => q(Panang),
   q(pgy) => q(Pongyong),
   q(pmu) => q(Mirpur Panjabi),
   q(ppr) => q(Piru),
   q(puz) => q(Purum Naga),
   q(sap) => q(Sanapana),
   q(sca) => q(Sansu),
   q(sgo) => q(Songa),
   q(tgg) => q(Tangga),
   q(thx) => q(The),
   q(tie) => q(Tingal),
   q(tkk) => q(Takpa),
   q(tlw) => q(South Wemale),
   q(tsf) => q(Southwestern Tamang),
   q(unp) => q(Worora),
   q(uok) => q(Uokha),
   q(wit) => q(Wintu),
   q(wiw) => q(Wirangu),
   q(xia) => q(Xiandao),
   q(xsj) => q(Subi),
   q(ybd) => q(Yangbye),
   q(yds) => q(Yiddish Sign Language),
   q(yen) => q(Yendang),
   q(yiy) => q(Yir Yoront),
   q(yma) => q(Yamphe),
   q(ymt) => q(Mator-Taygi-Karagas),
   q(ynh) => q(Yangho),
   q(yos) => q(Yos),
   q(yuu) => q(Yugh),
};

$Locale::Codes::Retired{'language'}{'term'}{'code'} = {
};

$Locale::Codes::Retired{'language'}{'alpha-2'}{'name'} = {
   q((afan) oromo) => [ q(om), q((Afan) Oromo) ],
   q(bengali; bangla) => [ q(bn), q(Bengali; Bangla) ],
   q(bhutani) => [ q(dz), q(Bhutani) ],
   q(bihari) => [ q(bh), q(Bihari) ],
   q(bokmal, norwegian) => [ q(nb), q(Bokmal, Norwegian) ],
   q(byelorussian) => [ q(be), q(Byelorussian) ],
   q(cambodian) => [ q(km), q(Cambodian) ],
   q(chichewa) => [ q(ny), q(Chichewa) ],
   q(chichewa; nyanja) => [ q(ny), q(Chichewa; Nyanja) ],
   q(divehi) => [ q(dv), q(Divehi) ],
   q(faeroese) => [ q(fo), q(Faeroese) ],
   q(fiji) => [ q(fj), q(Fiji) ],
   q(frisian) => [ q(fy), q(Frisian) ],
   q(gaelic) => [ q(gd), q(Gaelic) ],
   q(gaelic (scots)) => [ q(gd), q(Gaelic (Scots)) ],
   q(gallegan) => [ q(gl), q(Gallegan) ],
   q(greek) => [ q(el), q(Greek) ],
   q(greek, modern (1453-)) => [ q(el), q(Greek, Modern (1453-)) ],
   q(greenlandic) => [ q(kl), q(Greenlandic) ],
   q(interlingua) => [ q(ia), q(Interlingua) ],
   q(inupiak) => [ q(ik), q(Inupiak) ],
   q(khmer) => [ q(km), q(Khmer) ],
   q(kirundi) => [ q(rn), q(Kirundi) ],
   q(laothian) => [ q(lo), q(Laothian) ],
   q(latvian, lettish) => [ q(lv), q(Latvian, Lettish) ],
   q(letzeburgesch) => [ q(lb), q(Letzeburgesch) ],
   q(malay) => [ q(ms), q(Malay) ],
   q(marshall) => [ q(mh), q(Marshall) ],
   q(moldavian) => [ q(mo), q(Moldavian) ],
   q(ndebele, north) => [ q(nd), q(Ndebele, North) ],
   q(ndebele, south) => [ q(nr), q(Ndebele, South) ],
   q(nepali) => [ q(ne), q(Nepali) ],
   q(norwegian bokmal) => [ q(nb), q(Norwegian Bokmal) ],
   q(occitan) => [ q(oc), q(Occitan) ],
   q(oriya) => [ q(or), q(Oriya) ],
   q(ossetian; ossetic) => [ q(os), q(Ossetian; Ossetic) ],
   q(pashto, pushto) => [ q(ps), q(Pashto, Pushto) ],
   q(punjabi) => [ q(pa), q(Punjabi) ],
   q(rhaeto-romance) => [ q(rm), q(Rhaeto-Romance) ],
   q(sami) => [ q(se), q(Sami) ],
   q(sangro) => [ q(sg), q(Sangro) ],
   q(scots gaelic) => [ q(gd), q(Scots Gaelic) ],
   q(sesotho) => [ q(st), q(Sesotho) ],
   q(setswana) => [ q(tn), q(Setswana) ],
   q(singhalese) => [ q(si), q(Singhalese) ],
   q(sinhalese) => [ q(si), q(Sinhalese) ],
   q(siswati) => [ q(ss), q(Siswati) ],
   q(sotho) => [ q(st), q(Sotho) ],
   q(sotho, southern) => [ q(st), q(Sotho, Southern) ],
   q(swahili) => [ q(sw), q(Swahili) ],
   q(tegulu) => [ q(te), q(Tegulu) ],
   q(tonga) => [ q(to), q(Tonga) ],
   q(volapuk) => [ q(vo), q(Volapuk) ],
};

$Locale::Codes::Retired{'language'}{'alpha-3'}{'name'} = {
   q(!o!ung) => [ q(oun), q(!O!ung) ],
   q(=/kx'au//'ein) => [ q(aue), q(=/Kx'au//'ein) ],
   q(a'tong) => [ q(aot), q(A'tong) ],
   q(adap) => [ q(adp), q(Adap) ],
   q(aghu tharnggalu) => [ q(ggr), q(Aghu Tharnggalu) ],
   q(ainu) => [ q(ain), q(Ainu) ],
   q(amarag) => [ q(amg), q(Amarag) ],
   q(anu) => [ q(anl), q(Anu) ],
   q(apalik) => [ q(apo), q(Apalik) ],
   q(aramanik) => [ q(aam), q(Aramanik) ],
   q(aromanian) => [ q(rup), q(Aromanian) ],
   q(atong) => [ q(ato), q(Atong) ],
   q(ayi (china)) => [ q(ayx), q(Ayi (China)) ],
   q(bandjigali) => [ q(bjd), q(Bandjigali) ],
   q(bangi me) => [ q(dba), q(Bangi Me) ],
   q(bantu (other)) => [ q(bnt), q(Bantu (Other)) ],
   q(basa) => [ q(bas), q(Basa) ],
   q(bauro) => [ q(bxa), q(Bauro) ],
   q(bebe) => [ q(bzv), q(Bebe) ],
   q(bedik) => [ q(tnr), q(Bedik) ],
   q(begbere-ejar) => [ q(bqv), q(Begbere-Ejar) ],
   q(bekwil) => [ q(bkw), q(Bekwil) ],
   q(bemba) => [ q(bem), q(Bemba) ],
   q(bemba (democratic republic of congo)) => [ q(bmy), q(Bemba (Democratic Republic of Congo)) ],
   q(blin) => [ q(byn), q(Blin) ],
   q(borna (democratic republic of congo)) => [ q(bxx), q(Borna (Democratic Republic of Congo)) ],
   q(brek karen) => [ q(kvl), q(Brek Karen) ],
   q(burum-mindik) => [ q(bmu), q(Burum-Mindik) ],
   q(buya) => [ q(byy), q(Buya) ],
   q(central bicolano) => [ q(bcl), q(Central Bicolano) ],
   q(chaungtha) => [ q(ccq), q(Chaungtha) ],
   q(creoles and pidgins ) => [ q(crp), q(Creoles and pidgins ) ],
   q(creoles and pidgins, english based) => [ q(cpe), q(Creoles and pidgins, English based) ],
   q(creoles and pidgins, french-based ) => [ q(cpf), q(Creoles and pidgins, French-based ) ],
   q(creoles and pidgins, portuguese-based ) => [ q(cpp), q(Creoles and pidgins, Portuguese-based ) ],
   q(dakaka) => [ q(bpa), q(Dakaka) ],
   q(darling) => [ q(drl), q(Darling) ],
   q(daza) => [ q(dzd), q(Daza) ],
   q(dhangu) => [ q(dhg), q(Dhangu) ],
   q(dhanwar (india)) => [ q(dha), q(Dhanwar (India)) ],
   q(dimir) => [ q(dmc), q(Dimir) ],
   q(djangu) => [ q(dhg), q(Djangu) ],
   q(dogri) => [ q(doi), q(Dogri) ],
   q(duduela) => [ q(duk), q(Duduela) ],
   q(dumpu) => [ q(wtf), q(Dumpu) ],
   q(durango nahuatl) => [ q(nln), q(Durango Nahuatl) ],
   q(dutch, middle (ca.1050-1350)) => [ q(dum), q(Dutch, Middle (ca.1050-1350)) ],
   q(elpaputih) => [ q(elp), q(Elpaputih) ],
   q(emok) => [ q(emo), q(Emok) ],
   q(english, middle (1100-1500)) => [ q(enm), q(English, Middle (1100-1500)) ],
   q(english, old (ca.450-1100)) => [ q(ang), q(English, Old (ca.450-1100)) ],
   q(fang) => [ q(fan), q(Fang) ],
   q(forest maninka) => [ q(myq), q(Forest Maninka) ],
   q(french, middle (ca.1400-1600)) => [ q(frm), q(French, Middle (ca.1400-1600)) ],
   q(french, old (842-ca.1400)) => [ q(fro), q(French, Old (842-ca.1400)) ],
   q(galoli) => [ q(gal), q(Galoli) ],
   q(garawa) => [ q(gbc), q(Garawa) ],
   q(gbati-ri) => [ q(gti), q(Gbati-ri) ],
   q(gbaya) => [ q(gba), q(Gbaya) ],
   q(gelao) => [ q(gio), q(Gelao) ],
   q(german, middle high (ca.1050-1500)) => [ q(gmh), q(German, Middle High (ca.1050-1500)) ],
   q(german, old high (ca.750-1050)) => [ q(goh), q(German, Old High (ca.750-1050)) ],
   q(greek, ancient (to 1453)) => [ q(grc), q(Greek, Ancient (to 1453)) ],
   q(guwinmal) => [ q(xgm), q(Guwinmal) ],
   q(hainyaxo bozo) => [ q(bzx), q(Hainyaxo Bozo) ],
   q(hawai'i pidgin sign language) => [ q(hps), q(Hawai'i Pidgin Sign Language) ],
   q(horuru) => [ q(hrr), q(Horuru) ],
   q(hungworo) => [ q(nat), q(Hungworo) ],
   q(ibilo) => [ q(ibi), q(Ibilo) ],
   q(idun) => [ q(ldb), q(Idun) ],
   q(ikobi-mena) => [ q(meb), q(Ikobi-Mena) ],
   q(imeraguen) => [ q(ime), q(Imeraguen) ],
   q(irish, middle (900-1200)) => [ q(mga), q(Irish, Middle (900-1200)) ],
   q(irish, old (to 900)) => [ q(sga), q(Irish, Old (to 900)) ],
   q(izi-ezaa-ikwo-mgbo) => [ q(izi), q(Izi-Ezaa-Ikwo-Mgbo) ],
   q(jarawa (nigeria)) => [ q(jar), q(Jarawa (Nigeria)) ],
   q(kadara) => [ q(kad), q(Kadara) ],
   q(kado) => [ q(kdv), q(Kado) ],
   q(kag-fer-jiir-koor-ror-us-zuksun) => [ q(gel), q(Kag-Fer-Jiir-Koor-Ror-Us-Zuksun) ],
   q(kakauhua) => [ q(kbf), q(Kakauhua) ],
   q(kakihum) => [ q(kxe), q(Kakihum) ],
   q(kamba) => [ q(kam), q(Kamba) ],
   q(kemezung) => [ q(dmo), q(Kemezung) ],
   q(kenuzi-dongola) => [ q(kzh), q(Kenuzi-Dongola) ],
   q(khumi awa chin) => [ q(cka), q(Khumi Awa Chin) ],
   q(kol) => [ q(ekl), q(Kol) ],
   q(kolum so dogon) => [ q(dkl), q(Kolum So Dogon) ],
   q(konkani) => [ q(kok), q(Konkani) ],
   q(kumak) => [ q(nee), q(Kumak) ],
   q(kuman) => [ q(kue), q(Kuman) ],
   q(kutang ghale) => [ q(ght), q(Kutang Ghale) ],
   q(kwak) => [ q(kwq), q(Kwak) ],
   q(kwato) => [ q(kop), q(Kwato) ],
   q(kyenga) => [ q(tye), q(Kyenga) ],
   q(lama (myanmar)) => [ q(lay), q(Lama (Myanmar)) ],
   q(lamam) => [ q(lmm), q(Lamam) ],
   q(lengua) => [ q(leg), q(Lengua) ],
   q(lingkhim) => [ q(lii), q(Lingkhim) ],
   q(lowa) => [ q(loy), q(Lowa) ],
   q(lower tanudan kalinga) => [ q(kml), q(Lower Tanudan Kalinga) ],
   q(luhu) => [ q(lcq), q(Luhu) ],
   q(mahei) => [ q(mja), q(Mahei) ],
   q(malakhel) => [ q(mld), q(Malakhel) ],
   q(malasanga) => [ q(mqz), q(Malasanga) ],
   q(maligo) => [ q(mwj), q(Maligo) ],
   q(mangetti dune !xung) => [ q(gfx), q(Mangetti Dune !Xung) ],
   q(mann) => [ q(mev), q(Mann) ],
   q(mari) => [ q(chm), q(Mari) ],
   q(maskoy pidgin) => [ q(mhh), q(Maskoy Pidgin) ],
   q(mator-taygi-karagas) => [ q(ymt), q(Mator-Taygi-Karagas) ],
   q(maykulan) => [ q(mnt), q(Maykulan) ],
   q(mbabaram) => [ q(vmb), q(Mbabaram) ],
   q(mbedam) => [ q(xmd), q(Mbedam) ],
   q(mbu') => [ q(muc), q(Mbu') ],
   q(mea) => [ q(meg), q(Mea) ],
   q(mende) => [ q(men), q(Mende) ],
   q(minigir) => [ q(vmg), q(Minigir) ],
   q(mire) => [ q(mvh), q(Mire) ],
   q(miri) => [ q(mrg), q(Miri) ],
   q(mirpur panjabi) => [ q(pmu), q(Mirpur Panjabi) ],
   q(misima-paneati) => [ q(mpx), q(Misima-Paneati) ],
   q(mixifore) => [ q(mfg), q(Mixifore) ],
   q(mpongmpong) => [ q(mgg), q(Mpongmpong) ],
   q(mro chin) => [ q(cmr), q(Mro Chin) ],
   q(mudbura) => [ q(mwd), q(Mudbura) ],
   q(muduapa) => [ q(wiv), q(Muduapa) ],
   q(nakama) => [ q(nib), q(Nakama) ],
   q(nama (namibia)) => [ q(naq), q(Nama (Namibia)) ],
   q(nanggu) => [ q(ngr), q(Nanggu) ],
   q(nangikurrunggurr) => [ q(nam), q(Nangikurrunggurr) ],
   q(ndrag'ngith) => [ q(dgt), q(Ndrag'ngith) ],
   q(nepal bhasa) => [ q(new), q(Nepal Bhasa) ],
   q(nga la) => [ q(hlt), q(Nga La) ],
   q(ngalkbun) => [ q(ngk), q(Ngalkbun) ],
   q(ngong) => [ q(nnx), q(Ngong) ],
   q(ngura) => [ q(nbx), q(Ngura) ],
   q(nisi (india)) => [ q(dap), q(Nisi (India)) ],
   q(nootka) => [ q(noo), q(Nootka) ],
   q(norse, old) => [ q(non), q(Norse, Old) ],
   q(north wemale) => [ q(weo), q(North Wemale) ],
   q(northeast pashayi) => [ q(aee), q(Northeast Pashayi) ],
   q(northern bai) => [ q(bfc), q(Northern Bai) ],
   q(northern catanduanes bicolano) => [ q(cts), q(Northern Catanduanes Bicolano) ],
   q(northern lorung) => [ q(lbr), q(Northern Lorung) ],
   q(northwest pashayi) => [ q(glh), q(Northwest Pashayi) ],
   q(nsari) => [ q(asj), q(Nsari) ],
   q(nung (myanmar)) => [ q(nun), q(Nung (Myanmar)) ],
   q(omati) => [ q(mgx), q(Omati) ],
   q(ounge) => [ q(oue), q(Ounge) ],
   q(palu) => [ q(pbz), q(Palu) ],
   q(panang) => [ q(pcr), q(Panang) ],
   q(persian, old (ca.600-400 b.c.)) => [ q(peo), q(Persian, Old (ca.600-400 B.C.)) ],
   q(piru) => [ q(ppr), q(Piru) ],
   q(pongyong) => [ q(pgy), q(Pongyong) ],
   q(provencal, old (to 1500)) => [ q(pro), q(Provencal, Old (to 1500)) ],
   q(purum naga) => [ q(puz), q(Purum Naga) ],
   q(pyu) => [ q(pby), q(Pyu) ],
   q(sakam) => [ q(skm), q(Sakam) ],
   q(san marcos tlalcoyalco popoloca) => [ q(pls), q(San Marcos Tlalcoyalco Popoloca) ],
   q(sansu) => [ q(sca), q(Sansu) ],
   q(sara dunjo) => [ q(koj), q(Sara Dunjo) ],
   q(sauk) => [ q(skc), q(Sauk) ],
   q(sepen) => [ q(spm), q(Sepen) ],
   q(seraiki) => [ q(skr), q(Seraiki) ],
   q(shabo) => [ q(sbf), q(Shabo) ],
   q(shangzhai) => [ q(jih), q(Shangzhai) ],
   q(so (cameroon)) => [ q(sox), q(So (Cameroon)) ],
   q(songa) => [ q(sgo), q(Songa) ],
   q(south american indian (other)) => [ q(sai), q(South American Indian (Other)) ],
   q(south wemale) => [ q(tlw), q(South Wemale) ],
   q(southeast pashayi) => [ q(psi), q(Southeast Pashayi) ],
   q(southern catanduanes bicolano) => [ q(bln), q(Southern Catanduanes Bicolano) ],
   q(southern lorung) => [ q(lrr), q(Southern Lorung) ],
   q(southwest pashayi) => [ q(psh), q(Southwest Pashayi) ],
   q(southwestern tamang) => [ q(tsf), q(Southwestern Tamang) ],
   q(subi) => [ q(xsj), q(Subi) ],
   q(sulung) => [ q(suv), q(Sulung) ],
   q(takpa) => [ q(tkk), q(Takpa) ],
   q(talur) => [ q(ilw), q(Talur) ],
   q(tangga) => [ q(tgg), q(Tangga) ],
   q(tangkhul naga) => [ q(nmf), q(Tangkhul Naga) ],
   q(the) => [ q(thx), q(The) ],
   q(tina sambal) => [ q(xsb), q(Tina Sambal) ],
   q(tingal) => [ q(tie), q(Tingal) ],
   q(tsotsitaal) => [ q(fly), q(Tsotsitaal) ],
   q(turkish, ottoman (1500-1928)) => [ q(ota), q(Turkish, Ottoman (1500-1928)) ],
   q(unggarranggu) => [ q(xun), q(Unggarranggu) ],
   q(uokha) => [ q(uok), q(Uokha) ],
   q(upper tanudan kalinga) => [ q(kgh), q(Upper Tanudan Kalinga) ],
   q(vasekela bushman) => [ q(vaj), q(Vasekela Bushman) ],
   q(walamo) => [ q(wal), q(Walamo) ],
   q(walo kumbe dogon) => [ q(dwl), q(Walo Kumbe Dogon) ],
   q(waray) => [ q(war), q(Waray) ],
   q(welaung) => [ q(weu), q(Welaung) ],
   q(were) => [ q(wei), q(Were) ],
   q(western canadian inuktitut) => [ q(ikt), q(Western Canadian Inuktitut) ],
   q(worora) => [ q(unp), q(Worora) ],
   q(xiandao) => [ q(xia), q(Xiandao) ],
   q(yamphe) => [ q(yma), q(Yamphe) ],
   q(yangbye) => [ q(ybd), q(Yangbye) ],
   q(yangho) => [ q(ynh), q(Yangho) ],
   q(yeskwa) => [ q(yes), q(Yeskwa) ],
   q(yiddish sign language) => [ q(yds), q(Yiddish Sign Language) ],
   q(yonggom) => [ q(yon), q(Yonggom) ],
   q(yos) => [ q(yos), q(Yos) ],
   q(yuaga) => [ q(nua), q(Yuaga) ],
   q(yugh) => [ q(yuu), q(Yugh) ],
   q(zyphe) => [ q(zyp), q(Zyphe) ],
};

$Locale::Codes::Retired{'language'}{'term'}{'name'} = {
};


1;
